@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2022 Joel Rosdahl and other contributors
+// Copyright (C) 2020-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -18,8 +18,14 @@
 
 #pragma once
 
+#include <Fd.hpp>
+#include <hashutil.hpp>
+#include <util/Duration.hpp>
+#include <util/TimePoint.hpp>
+
 #include <cstdint>
 #include <functional>
+#include <optional>
 #include <string>
 
 class Config;
@@ -41,7 +47,26 @@ public:
     checked_for_temporal_macros = 1,
   };
 
-  InodeCache(const Config& config);
+  // `min_age` specifies how old a file must be to be put in the cache. The
+  // reason for this is that there is a race condition that consists of these
+  // events:
+  //
+  // 1. A file is written with content C1, size S and timestamp (ctime/mtime) T.
+  // 2. Ccache hashes the file content and asks the inode cache to store the
+  //    digest with a hash of S and T (and some other data) as the key.
+  // 3. The file is quickly thereafter written with content C2 without changing
+  //    size S and timestamp T. The timestamp is not updated since the file
+  //    writes are made within a time interval smaller than the granularity of
+  //    the clock used for file system timestamps. At the time of writing, a
+  //    common granularity on a Linux system is 0.004 s (250 Hz).
+  // 4. The inode cache is asked for the file digest and the inode cache
+  //    delivers a digest of C1 even though the file's content is C2.
+  //
+  // To avoid the race condition, the inode cache only caches inodes whose
+  // timestamp was updated more than `min_age` ago. The default value is a
+  // conservative 2 seconds since not all file systems have subsecond
+  // resolution.
+  InodeCache(const Config& config, util::Duration min_age = util::Duration(2));
   ~InodeCache();
 
   // Return whether it's possible to use the inode cache on the filesystem
@@ -53,10 +78,8 @@ public:
   //
   // Returns true if saved values could be retrieved from the cache, false
   // otherwise.
-  bool get(const std::string& path,
-           ContentType type,
-           Digest& file_digest,
-           int* return_value = nullptr);
+  std::optional<HashSourceCodeResult>
+  get(const std::string& path, ContentType type, Digest& file_digest);
 
   // Put hash digest and return value from a successful call to do_hash_file()
   // in hashutil.cpp.
@@ -65,7 +88,7 @@ public:
   bool put(const std::string& path,
            ContentType type,
            const Digest& file_digest,
-           int return_value = 0);
+           HashSourceCodeResult return_value);
 
   // Unmaps the current cache and removes the mapped file from disk.
   //
@@ -101,14 +124,17 @@ private:
   using BucketHandler = std::function<void(Bucket* bucket)>;
 
   bool mmap_file(const std::string& inode_cache_file);
-  static bool
-  hash_inode(const std::string& path, ContentType type, Digest& digest);
+  bool hash_inode(const std::string& path, ContentType type, Digest& digest);
   bool with_bucket(const Digest& key_digest,
                    const BucketHandler& bucket_handler);
   static bool create_new_file(const std::string& filename);
   bool initialize();
 
   const Config& m_config;
+  util::Duration m_min_age;
+  Fd m_fd;
   struct SharedRegion* m_sr = nullptr;
   bool m_failed = false;
+  const pid_t m_self_pid;
+  util::TimePoint m_last_fs_space_check;
 };
