@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2022 Joel Rosdahl and other contributors
+// Copyright (C) 2019-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -20,6 +20,7 @@
 
 #include <core/wincompat.hpp>
 #include <util/TimePoint.hpp>
+#include <util/file.hpp>
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -116,6 +117,9 @@ public:
   // otherwise false.
   operator bool() const;
 
+  // Return the path that this stat result refers to.
+  const std::string& path() const;
+
   // Return whether this object refers to the same device and i-node as `other`
   // does.
   bool same_inode_as(const Stat& other) const;
@@ -148,6 +152,7 @@ protected:
   Stat(StatFunction stat_function, const std::string& path, OnError on_error);
 
 private:
+  std::string m_path;
   stat_t m_stat;
   int m_errno;
 
@@ -170,6 +175,12 @@ Stat::same_inode_as(const Stat& other) const
   return m_errno == 0 && device() == other.device() && inode() == other.inode();
 }
 
+inline const std::string&
+Stat::path() const
+{
+  return m_path;
+}
+
 inline int
 Stat::error_number() const
 {
@@ -240,7 +251,7 @@ inline uint64_t
 Stat::size_on_disk() const
 {
 #ifdef _WIN32
-  return (size() + 1023) & ~1023;
+  return util::likely_size_on_disk(size());
 #else
   return m_stat.st_blocks * 512;
 #endif
