@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2022 Joel Rosdahl and other contributors
+// Copyright (C) 2019-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -438,7 +438,8 @@ expand_environment_variables(const std::string& str)
 {
   std::string result;
   const char* left = str.c_str();
-  for (const char* right = left; *right; ++right) {
+  const char* right = left;
+  while (*right) {
     if (*right == '$') {
       result.append(left, right - left);
 
@@ -471,6 +472,7 @@ expand_environment_variables(const std::string& str)
         left = right + 1;
       }
     }
+    ++right;
   }
   result += left;
   return result;
@@ -480,8 +482,13 @@ int
 fallocate(int fd, long new_size)
 {
 #ifdef HAVE_POSIX_FALLOCATE
-  return posix_fallocate(fd, 0, new_size);
-#else
+  const int posix_fallocate_err = posix_fallocate(fd, 0, new_size);
+  if (posix_fallocate_err == 0 || posix_fallocate_err != EINVAL) {
+    return posix_fallocate_err;
+  }
+  // the underlying filesystem does not support the operation so fallback to
+  // lseeks
+#endif
   off_t saved_pos = lseek(fd, 0, SEEK_END);
   off_t old_size = lseek(fd, 0, SEEK_END);
   if (old_size == -1) {
@@ -508,7 +515,6 @@ fallocate(int fd, long new_size)
   lseek(fd, saved_pos, SEEK_SET);
   free(buf);
   return err;
-#endif
 }
 
 std::string
@@ -519,9 +525,11 @@ format_argv_for_logging(const char* const* argv)
     if (i != 0) {
       result += ' ';
     }
-    for (const char* arg = argv[i]; *arg; ++arg) {
-      result += *arg;
+    std::string arg(argv[i]);
+    if (arg.empty() || arg.find(' ') != std::string::npos) {
+      arg = FMT("\"{}\"", arg);
     }
+    result += arg;
   }
   return result;
 }
@@ -549,30 +557,6 @@ format_base32hex(const uint8_t* data, size_t size)
   return result;
 }
 
-std::string
-format_human_readable_size(uint64_t size)
-{
-  if (size >= 1000 * 1000 * 1000) {
-    return FMT("{:.1f} GB", size / ((double)(1000 * 1000 * 1000)));
-  } else if (size >= 1000 * 1000) {
-    return FMT("{:.1f} MB", size / ((double)(1000 * 1000)));
-  } else {
-    return FMT("{:.1f} kB", size / 1000.0);
-  }
-}
-
-std::string
-format_parsable_size_with_suffix(uint64_t size)
-{
-  if (size >= 1000 * 1000 * 1000) {
-    return FMT("{:.1f}G", size / ((double)(1000 * 1000 * 1000)));
-  } else if (size >= 1000 * 1000) {
-    return FMT("{:.1f}M", size / ((double)(1000 * 1000)));
-  } else {
-    return FMT("{}", size);
-  }
-}
-
 void
 ensure_dir_exists(std::string_view dir)
 {
@@ -843,9 +827,9 @@ make_relative_path(const std::string& base_dir,
   const auto path_suffix = std::string(original_path.substr(path.length()));
   const auto real_path = Util::real_path(std::string(path));
 
-  const auto add_relpath_candidates = [&](auto path) {
+  const auto add_relpath_candidates = [&](auto p) {
     const std::string normalized_path =
-      Util::normalize_abstract_absolute_path(path);
+      Util::normalize_abstract_absolute_path(p);
     relpath_candidates.push_back(
       Util::get_relative_path(actual_cwd, normalized_path));
     if (apparent_cwd != actual_cwd) {
@@ -893,20 +877,14 @@ matches_dir_prefix_or_file(std::string_view dir_prefix_or_file,
              || is_dir_separator(dir_prefix_or_file.back()));
 }
 
-std::string
-normalize_abstract_absolute_path(std::string_view path)
+static std::string
+do_normalize_abstract_absolute_path(std::string_view path)
 {
   if (!util::is_absolute_path(path)) {
     return std::string(path);
   }
 
 #ifdef _WIN32
-  if (path.find("\\") != std::string_view::npos) {
-    std::string new_path(path);
-    std::replace(new_path.begin(), new_path.end(), '\\', '/');
-    return normalize_abstract_absolute_path(new_path);
-  }
-
   std::string drive(path.substr(0, 2));
   path = path.substr(2);
 #endif
@@ -953,11 +931,24 @@ normalize_abstract_absolute_path(std::string_view path)
 #endif
 }
 
+std::string
+normalize_abstract_absolute_path(std::string_view path)
+{
+#ifdef _WIN32
+  std::string new_path(path);
+  std::replace(new_path.begin(), new_path.end(), '\\', '/');
+  return do_normalize_abstract_absolute_path(new_path);
+#else
+  return do_normalize_abstract_absolute_path(path);
+#endif
+}
+
 std::string
 normalize_concrete_absolute_path(const std::string& path)
 {
   const auto normalized_path = normalize_abstract_absolute_path(path);
-  return Stat::stat(normalized_path).same_inode_as(Stat::stat(path))
+  return (normalized_path == path
+          || Stat::stat(normalized_path).same_inode_as(Stat::stat(path)))
            ? normalized_path
            : path;
 }
@@ -989,47 +980,6 @@ parse_duration(std::string_view duration)
   }
 }
 
-uint64_t
-parse_size(const std::string& value)
-{
-  errno = 0;
-
-  char* p;
-  double result = strtod(value.c_str(), &p);
-  if (errno != 0 || result < 0 || p == value.c_str() || value.empty()) {
-    throw core::Error(FMT("invalid size: \"{}\"", value));
-  }
-
-  while (isspace(*p)) {
-    ++p;
-  }
-
-  if (*p != '\0') {
-    unsigned multiplier = *(p + 1) == 'i' ? 1024 : 1000;
-    switch (*p) {
-    case 'T':
-      result *= multiplier;
-      [[fallthrough]];
-    case 'G':
-      result *= multiplier;
-      [[fallthrough]];
-    case 'M':
-      result *= multiplier;
-      [[fallthrough]];
-    case 'K':
-    case 'k':
-      result *= multiplier;
-      break;
-    default:
-      throw core::Error(FMT("invalid size: \"{}\"", value));
-    }
-  } else {
-    // Default suffix: G.
-    result *= 1000 * 1000 * 1000;
-  }
-  return static_cast<uint64_t>(result);
-}
-
 #ifndef _WIN32
 std::string
 read_link(const std::string& path)
